@@ -164,13 +164,11 @@ Return point or nil if point did not change or if no subtitle end
 can be found.  Use the format-specific function for MAJOR-MODE."
   (let ((orig-point (point)))
     (subed-jump-to-subtitle-text sub-id)
-    ;; Look for next separator or end of buffer.  We can't use
-    ;; `subed-srt--regexp-separator' here because if subtitle text is empty,
-    ;; it may be the only empty line in the separator, i.e. there's only one
-    ;; "\n".
-    (let ((regex (concat "\\([[:blank:]]*\n+[0-9]+\n\\|\\([[:blank:]]*\n*\\)\\'\\)")))
-      (when (re-search-forward regex nil t)
-        (goto-char (match-beginning 0))))
+    (unless (looking-at "\n") ; handle empty subtitle text
+      ;; Look for next separator or end of buffer.
+      (let ((regex (concat "\\(" subed--regexp-separator "[0-9]+\n\\|\\([[:blank:]]*\n*\\)\\'\\)")))
+        (when (re-search-forward regex nil t)
+          (goto-char (match-beginning 0)))))
     (unless (= (point) orig-point)
       (point))))
 
@@ -178,8 +176,12 @@ can be found.  Use the format-specific function for MAJOR-MODE."
   "Move point to next subtitle's ID.
 Return point or nil if there is no next subtitle.
 Use the format-specific function for MAJOR-MODE."
-  (when (re-search-forward (concat subed--regexp-separator "[0-9]+\n") nil t)
-    (subed-jump-to-subtitle-id)))
+  (let ((pos (point)))
+    (when (and (bolp) (> (point) (point-min))) (forward-char -1))
+    (if (re-search-forward (concat subed--regexp-separator "[0-9]+\n") nil t)
+        (subed-jump-to-subtitle-id)
+      (goto-char pos)
+      nil)))
 
 (cl-defmethod subed--backward-subtitle-id (&context (major-mode subed-srt-mode))
   "Move point to previous subtitle's ID.
@@ -281,8 +283,10 @@ Use the format-specific function for MAJOR-MODE."
               (setq new-end (subed-timestamp-to-msecs (match-string 0))))
             (subed-jump-to-subtitle-text)
             (delete-region pos (point))
-            (insert "\n")
-            (subed-set-subtitle-time-stop new-end)
+            (insert " ")
+            (let ((subed-enforce-time-boundaries nil))
+              (subed-set-subtitle-time-stop new-end))
+            (run-hooks 'subed-subtitle-merged-hook)
             (subed-regenerate-ids-soon))
         (error "No subtitle to merge into")))))
 
