@@ -43,19 +43,32 @@
 
 ;;; Parsing
 
-(defconst subed-vtt--regexp-timestamp "\\(\\([0-9]+\\):\\)?\\([0-9]+\\):\\([0-9]+\\)\\(?:\\.\\([0-9]+\\)\\)?")
-(defconst subed-vtt--regexp-separator "\\(?:[[:blank:]]*\n\n+NOTE[ \n]\\(?:.+?\n\\)+\\)*\n\n+")
+(defconst subed-vtt--regexp-timestamp "\\(?:\\(?:[0-9]+\\):\\)?\\(?:[0-9]+\\):\\(?:[0-9]+\\)\\(?:\\.\\(?:[0-9]+\\)\\)?")
+(defconst subed-vtt--regexp-separator "\\(?:\\(?:[ \t]*\n\\)+\\(?:NOTE[ \t\n]*[ \t]*\n[ \t]*\n\\)?\\)"
+  "Blank lines and possibly a comment.")
+(defconst subed-vtt--regexp-blank-separator "\\(?:[ \t]*\n[ \t]*\n\\|\\`\\(?:[ \t\n]*\\)\\)")
+(defconst subed-vtt--regexp-note "\\(NOTE[ \t\n]\\)")
 (defconst subed-vtt--regexp-identifier
   ;; According to https://developer.mozilla.org/en-US/docs/Web/API/WebVTT_API
   ;; Cues can start with an identifier which is a non empty line that does
   ;; not contain "-->".
-  "^[ \t]*[^ \t\n-]\\(?:[^\n-]\\|-[^\n-]\\|--[^\n>]\\)*[ \t]*\n")
+  "[ \t]*[^ \t\n-]\\(?:[^\n-]\\|-[^\n-]\\|--[^\n>]\\)*[ \t]*")
+(defconst subed-vtt--regexp-start-of-line "\\(?:\\`\\|\n\\)")
+(defconst subed-vtt--regexp-timing
+  (concat "[ \t]*" subed-vtt--regexp-timestamp "[ \t]*-->[ \t]*" subed-vtt--regexp-timestamp))
+(defconst subed-vtt--regexp-maybe-identifier-and-timing
+  (format "%s\\(%s%s\\|%s\\)"
+          subed-vtt--regexp-blank-separator
+          subed-vtt--regexp-identifier
+          subed-vtt--regexp-timing
+          subed-vtt--regexp-timing))
 
 (cl-defmethod subed--timestamp-to-msecs (time-string &context (major-mode subed-vtt-mode))
   "Find HH:MM:SS.MS pattern in TIME-STRING and convert it to milliseconds.
 Return nil if TIME-STRING doesn't match the pattern.
 Use the format-specific function for MAJOR-MODE."
-  (when (string-match subed--regexp-timestamp time-string)
+  (when (string-match "\\(\\([0-9]+\\):\\)?\\([0-9]+\\):\\([0-9]+\\)\\(?:\\.\\([0-9]+\\)\\)?"
+                      time-string)
     (let ((hours (string-to-number (or (match-string 2 time-string) "0")))
           (mins  (string-to-number (match-string 3 time-string)))
           (secs  (string-to-number (match-string 4 time-string)))
@@ -76,7 +89,11 @@ Use the format-specific function for MAJOR-MODE."
 Use the format-specific function for MAJOR-MODE."
   (save-excursion
     (when (subed-jump-to-subtitle-id)
-      (match-string 1))))
+      (cond
+       ((looking-at (concat "[ \t]*\\(" subed-vtt--regexp-timestamp "\\)"))
+        (match-string 1))
+       ((looking-at subed-vtt--regexp-identifier)
+        (string-trim (match-string 0)))))))
 
 (cl-defmethod subed--subtitle-id-at-msecs (msecs &context (major-mode subed-vtt-mode))
   "Return the ID of the subtitle at MSECS milliseconds.
@@ -84,6 +101,8 @@ Return nil if there is no subtitle at MSECS.  Use the
 format-specific function for MAJOR-MODE."
   (save-excursion
     (goto-char (point-min))
+    (unless (subed-subtitle-id)
+      (subed-forward-subtitle-time-start))
     ;; Move to first subtitle that starts at or after MSECS
     (catch 'subtitle-id
       (while (<= (or (subed-subtitle-msecs-start) -1) msecs)
@@ -96,17 +115,73 @@ format-specific function for MAJOR-MODE."
 
 ;;; Traversing
 
+(cl-defmethod subed--in-header-p (&context (major-mode subed-vtt-mode))
+  "Return non-nil if the point is in the file header.
+Use the format-specific function for MAJOR-MODE."
+  (save-excursion
+    (let ((orig-point (point)))
+      (goto-char (line-beginning-position))
+      (cond
+       ((and (looking-at subed-vtt--regexp-timing)
+             (looking-back subed-vtt--regexp-start-of-line))
+        nil)
+       ;; not looking right at a timestamp; check the previous line to see if it's blank
+       ((and
+         (looking-back subed-vtt--regexp-blank-separator)
+         (looking-at (format "[ \t]*\\(%s\n%s\\|%s\\)"
+                             subed-vtt--regexp-identifier
+                             subed-vtt--regexp-timing
+                             subed-vtt--regexp-note)))
+        nil)
+       ((re-search-backward (concat "^" subed-vtt--regexp-timing) nil t) nil)
+       ((re-search-backward (concat subed-vtt--regexp-blank-separator subed-vtt--regexp-note) nil t) nil)
+       (t t)))))
+
+(cl-defmethod subed--in-comment-p (&context (major-mode subed-vtt-mode))
+  "Return non-nil if the point is in a comment.
+Use the format-specific function for MAJOR-MODE."
+  (save-excursion
+    (goto-char (line-beginning-position))
+    (cond
+     ((looking-at subed-vtt--regexp-timing)
+      nil)
+     ((and (looking-at (format "%s\n%s"
+                               subed-vtt--regexp-identifier
+                               subed-vtt--regexp-timing))
+           (looking-back subed-vtt--regexp-blank-separator))
+      nil)
+     ;; looking at a comment
+     ((and
+       (looking-back "^[ \t]*\n\\|\\`\\(?:[ \t\n]*\\)")
+       (looking-at (concat "[ \t]*" subed-vtt--regexp-note)))
+      t)
+     ((re-search-backward (format "\\(%s%s\\)\\|%s%s"
+                                  subed-vtt--regexp-blank-separator
+                                  subed-vtt--regexp-note
+                                  subed-vtt--regexp-start-of-line
+                                  subed-vtt--regexp-timing)
+                          nil t)
+      ;;  found the note instead of the timestamp
+      (when (match-string 1)
+        t)))))
+
 (cl-defmethod subed--jump-to-subtitle-id (&context (major-mode subed-vtt-mode) &optional sub-id)
   "Move to the ID of a subtitle and return point.
 If SUB-ID is not given, focus the current subtitle's ID.
 Return point or nil if no subtitle ID could be found.
 WebVTT IDs are optional.  If an ID is not specified, use the timestamp instead.
 Use the format-specific function for MAJOR-MODE."
-  (let ((orig-point (point)) found)
+  (let ((orig-point (point))
+        found
+        (timestamp-line
+         (format "\\(?:\\`\\|\n[ \t]*\n+\\)\\(\\(%s\\)\n%s\\|\\(%s\\)\\)"
+                 subed-vtt--regexp-identifier
+                 subed-vtt--regexp-timing
+                 subed-vtt--regexp-timing)))
     (if (stringp sub-id)
         ;; Look for a line that contains the timestamp, preceded by one or more
         ;; blank lines or the beginning of the buffer.
-        (let* ((regex (concat "\\(" subed--regexp-separator "\\|\\`\\)\\("
+        (let* ((regex (concat "\\(" subed-vtt--regexp-blank-separator "\\|\\`\\)\\("
                               (regexp-quote sub-id) "\\)[ \n]")))
           (goto-char (point-min))
           (setq found (re-search-forward regex nil t))
@@ -114,31 +189,42 @@ Use the format-specific function for MAJOR-MODE."
               (goto-char (match-beginning 2))
             (goto-char orig-point)
             nil))
-      ;; Find two or more blank lines or the beginning of the buffer, followed
-      ;; by an optional line and a timestamp.
-      (or (and (re-search-backward subed--regexp-separator nil t)
-               (goto-char (match-end 0)))
-          (goto-char (point-min)))
-      (cond
-       ((looking-at (concat "\\(" subed-vtt--regexp-timestamp "\\) *--> *" subed-vtt--regexp-timestamp "\\(?:\s+.+\\)?\n"))
-        ;; no ID, use the timestamp
-        (point))
-       ((looking-at (concat "\\(.*\\)\n" subed-vtt--regexp-timestamp " *--> *" subed-vtt--regexp-timestamp "\\(?:\s+.+\\)?\n"))
-        (point))
-       ((looking-at "^NOTE[ \n]")
-        ;; At a subtitle's comment; scan forward for the timestamp
-        (if (re-search-forward
-             (concat
-              subed--regexp-separator
-              (concat "\\(.*\n\\)?\\(" subed-vtt--regexp-timestamp " *--> *" subed-vtt--regexp-timestamp "\\)\\(?:\s+.+\\)?*\n")) nil t)
-            (progn
-              (goto-char (or (match-beginning 1) (match-beginning 2) (point)))
-              (point))
-          (goto-char orig-point)
-          nil))
-       (t
-        (goto-char orig-point)
-        nil)))))
+      (catch 'done
+        (goto-char (line-beginning-position))
+        (cond
+         ((subed-in-header-p) (throw 'done nil))
+         ;; are we looking at a timestamp? Stay here, check for ID later
+         ((looking-at subed-vtt--regexp-timing) (point))
+         ;; are we at an ID? return that
+         ((and (or (looking-back subed-vtt--regexp-blank-separator) (bobp))
+               (not (looking-at "[ \t]*\n"))
+               (not (save-excursion (re-search-forward "-->" (line-end-position) t)))
+               (save-excursion
+                 (forward-line)
+                 (looking-at subed-vtt--regexp-timing)))
+          (throw 'done (point)))
+         ;; are we in a comment? Search forward
+         ((subed-in-comment-p)
+          (if (re-search-forward (concat subed-vtt--regexp-start-of-line "\\(" subed-vtt--regexp-timing "\\)") nil t)
+              (goto-char (match-beginning 1))
+            (throw 'done nil)))
+         ;; scan backwards for a timestamp
+         ((re-search-backward (concat subed-vtt--regexp-start-of-line "\\(" subed-vtt--regexp-timing "\\)") nil t)
+          (goto-char (match-beginning 1))))
+        ;; We are at a timestamp; check backwards for an ID
+        (cond
+         ((bobp) (throw 'done (point)))
+         ((progn
+            (previous-line)
+            (goto-char (line-beginning-position))
+            (and (or (looking-back subed-vtt--regexp-blank-separator) (bobp))
+                 (not (looking-at "[ \t]*\n"))
+                 (save-excursion (not (re-search-forward "-->" (line-end-position) t)))))
+          (throw 'done (point)))
+         (t
+          ;; go back
+          (forward-line 1)
+          (point)))))))
 
 (cl-defmethod subed--jump-to-subtitle-time-start (&context (major-mode subed-vtt-mode) &optional sub-id)
   "Move point to subtitle's start time.
@@ -146,9 +232,9 @@ If SUB-ID is not given, use subtitle on point.
 Return point or nil if no start time could be found.
 Use the format-specific function for MAJOR-MODE."
   (when (subed-jump-to-subtitle-id sub-id)
-    (if (looking-at subed--regexp-timestamp)
+    (if (looking-at subed-vtt--regexp-timing)
         (point)
-      (when (re-search-forward subed--regexp-timestamp nil t)
+      (when (re-search-forward subed-vtt--regexp-timing nil t)
         (goto-char (match-beginning 0))
         (point)))))
 
@@ -158,7 +244,7 @@ If SUB-ID is not given, use subtitle on point.
 Return point or nil if no stop time could be found.
 Use the format-specific function for MAJOR-MODE."
   (when (subed-jump-to-subtitle-time-start sub-id)
-    (re-search-forward " *--> *" (line-end-position) t)
+    (re-search-forward "[ \t]*-->[ \t]*" (line-end-position) t)
     (when (looking-at subed--regexp-timestamp)
       (point))))
 
@@ -176,36 +262,67 @@ Use the format-specific function for MAJOR-MODE."
 If SUB-ID is not given, use subtitle on point.
 Return point or nil if point did not change or if no subtitle end
 can be found.  Use the format-specific function for MAJOR-MODE."
-  (let ((orig-point (point)))
-    (subed-jump-to-subtitle-text sub-id)
-    ;; Look for next separator or end of buffer.
-    (let ((regex subed-vtt--regexp-separator))
-      (if (eolp)
-          (unless (= (point) orig-point)
-            (point))
-        (if (re-search-forward regex nil t)
-            (goto-char (match-beginning 0))
-          (goto-char (point-max))
-          (skip-syntax-backward " "))
-        (unless (= (point) orig-point)
-          (point))))))
+  (let* ((orig-point (point))
+				 (case-fold-search nil)
+         text-point)
+    (catch 'done
+      ;; go back to the text
+      (if (subed-in-comment-p)
+          (if (re-search-forward (concat subed-vtt--regexp-start-of-line subed-vtt--regexp-timing) nil t)
+              (progn
+                (unless (eobp) (forward-line 1) (goto-char (line-beginning-position)))
+                (setq text-point (point)))
+            (throw 'done nil))
+        (setq text-point
+              (or (subed-jump-to-subtitle-text sub-id)
+                  (point))))
+      (goto-char (line-beginning-position))
+
+      (cond
+       ;; are we in the header?
+       ((subed-in-header-p)
+        (goto-char orig-point)
+        (throw 'done nil))
+       ;; are we looking at the next timing?
+       ((looking-at (concat "[ \t\n]*" subed-vtt--regexp-timing)) (point))
+       ;; are we looking at an ID and then the next timing?
+       ((looking-at (format "\\(?:[ \t]*\n\\)+%s\n%s"
+                            subed-vtt--regexp-identifier
+                            subed-vtt--regexp-timing))
+        (point))
+       ((re-search-forward (format "\\(%s%s\\)\\|\\(%s%s\n%s\\)\\|\\(%s%s\\)\\|%s.*-->"
+                                   subed-vtt--regexp-blank-separator
+                                   subed-vtt--regexp-note
+                                   subed-vtt--regexp-blank-separator
+                                   subed-vtt--regexp-identifier
+                                   subed-vtt--regexp-timing
+                                   subed-vtt--regexp-start-of-line
+                                   subed-vtt--regexp-timing
+                                   subed-vtt--regexp-start-of-line)
+                           nil t)
+        (goto-char (max text-point (match-beginning 0))))
+       (t (goto-char (point-max))))
+      (skip-chars-backward " \t\n")
+      (goto-char (max (point) text-point))
+      (unless (= (point) orig-point)
+        (point)))))
 
 (cl-defmethod subed--forward-subtitle-id (&context (major-mode subed-vtt-mode))
   "Move point to next subtitle's ID.
 Return point or nil if there is no next subtitle.  Use the
 format-specific function for MAJOR-MODE."
   (let ((orig-point (point)))
-    (subed-jump-to-subtitle-end)
-    (when (and (bolp) (> (point) (point-min))) (forward-char -1))
-    (if (re-search-forward (concat subed--regexp-separator
-                                   "\\(.*?\n\\)?"
-                                   subed--regexp-timestamp
-                                   " *--> *" subed--regexp-timestamp)
+    (when (subed-subtitle-id)
+      (subed-jump-to-subtitle-end))
+    (if (re-search-forward (format "\\(?:%s\\(%s\n%s\\)\\|%s\\(%s\\)\\)"
+                                   subed-vtt--regexp-blank-separator
+                                   subed-vtt--regexp-identifier
+                                   subed-vtt--regexp-timing
+                                   subed-vtt--regexp-start-of-line
+                                   subed-vtt--regexp-timing)
                            nil t)
-        (or (subed-jump-to-subtitle-id)
-            (progn
-              (goto-char orig-point)
-              nil))
+        (goto-char (or (match-beginning 1)
+                       (match-beginning 2)))
       (goto-char orig-point)
       nil)))
 
@@ -249,9 +366,11 @@ Return point, or nil if no comment could be found.
 Use the format-specific function for MAJOR-MODE."
   (let ((pos (point)))
     (if (and (subed-jump-to-subtitle-id sub-id)
-             (re-search-backward "^NOTE"
-                                 (or (save-excursion (subed-backward-subtitle-end)) (point-min)) t))
-        (progn (goto-char (match-beginning 0))
+             (re-search-backward
+              (concat subed-vtt--regexp-blank-separator
+                      "\\(NOTE[ \t\n]\\)")
+              (or (save-excursion (subed-backward-subtitle-end)) (point-min)) t))
+        (progn (goto-char (match-beginning 1))
                (point))
       (goto-char pos)
       nil)))
@@ -315,8 +434,9 @@ point.  Use the format-specific function for MAJOR-MODE."
   (insert (subed-make-subtitle id start stop text comment))
   (when (looking-at (concat "\\([[:space:]]*\\|^\\)" subed--regexp-timestamp))
     (insert "\n"))
-  (forward-line -2)
-  (subed-jump-to-subtitle-text))
+  (skip-chars-backward " \t\n")
+  (subed-jump-to-subtitle-text)
+  (point))
 
 (cl-defmethod subed--append-subtitle (&context (major-mode subed-vtt-mode) &optional id start stop text comment)
   "Insert new subtitle after the subtitle at point.
@@ -350,14 +470,23 @@ Update the end timestamp accordingly.
 Use the format-specific function for MAJOR-MODE."
   (save-excursion
     (subed-jump-to-subtitle-end)
-    (let ((pos (point)) new-end)
+    (let ((pos (point))
+          cur-comment
+          next-comment
+          new-end)
       (if (subed-forward-subtitle-time-stop)
           (progn
+            (setq next-comment (subed-subtitle-comment))
             (when (looking-at subed--regexp-timestamp)
               (setq new-end (subed-timestamp-to-msecs (match-string 0))))
             (subed-jump-to-subtitle-text)
             (delete-region pos (point))
             (insert " ")
+            (when next-comment
+              (setq cur-comment (subed-subtitle-comment))
+              (subed-set-subtitle-comment (if cur-comment
+                                              (concat cur-comment "\n" next-comment)
+                                            next-comment)))
             (let ((subed-enforce-time-boundaries nil))
               (subed-set-subtitle-time-stop new-end))
             (run-hooks 'subed-subtitle-merged-hook))
