@@ -78,8 +78,47 @@ Return a list of ((start . ?), (end . ?) (text . ?))."
                    rec))
                text-elements))))
 
+(defun subed-word-data--extract-words-from-youtube-vtt (file &optional from-string)
+  "Extract the timing from FILE which is a VTT from YouTube.
+Return a list of ((start . ?), (end . ?) (text . ?)).
+If FROM-STRING is non-nil, treat FILE as the data itself."
+  (with-temp-buffer
+    (subed-vtt-mode)
+    (if from-string
+        (insert file)
+      (insert-file-contents file))
+    (let ((list (subed-subtitle-list))
+          results
+          s
+          start
+          stop
+          i)
+      (dolist (sub list)
+        (when (string-match "<c>" (elt sub 3))
+          (setq s (elt sub 3))
+          (setq i 0)
+          (setq start (elt sub 1))
+          (while (and (< i (length s))
+                      (string-match "\\(.+?\\)<\\([0-9]+:[0-9]+:[0-9]+\\.[0-9]+\\)>" s i))
+            (setq stop (1- (save-match-data (subed-timestamp-to-msecs (match-string 2 s)))))
+            (push `((text . ,(save-match-data
+                               (string-trim (replace-regexp-in-string "</?c>" "" (match-string 1 s)))))
+                    (start . ,start)
+                    (end . ,stop))
+                  results)
+            (setq i (match-end 0)
+                  start (1+ stop)))
+          (if (and (< i (length s))
+                   (not (string= "" (string-trim (substring s i)))))
+              (push `((text . ,(string-trim
+                              (save-match-data (replace-regexp-in-string "</?c>" "" (substring s i)))))
+                      (start . ,start)
+                      (end . ,(elt sub 2)))
+                    results))))
+      (nreverse results))))
+
 (defun subed-word-data--extract-words-from-whisperx-json (file &optional from-string)
-  "Extract the timing from file in WhisperX's JSON format.
+  "Extract the timing from FILE in WhisperX's JSON format.
 Return a list of ((start . ?), (end . ?) (text . ?) (score . ?)).
 If FROM-STRING is non-nil, treat FILE as the data itself."
   (let* ((json-object-type 'alist)
@@ -113,16 +152,18 @@ If FROM-STRING is non-nil, treat FILE as the data itself."
 
 (defun subed-word-data--load (data)
   "Load word-level timing from DATA.
-For now, only SRV2 files are supported."
-  (setq-local subed-word-data--cache data)
-  (add-hook 'subed-split-subtitle-timestamp-functions #'subed-word-data-split-at-word-timestamp -5 t)
-  (subed-word-data-refresh-text-properties))
+Supports WhisperX JSON, YouTube VTT, and Youtube SRV2 files."
+  (when data
+    (setq-local subed-word-data--cache data)
+    (add-hook 'subed-split-subtitle-timestamp-functions #'subed-word-data-split-at-word-timestamp -5 t)
+    (subed-word-data-refresh-text-properties)
+    data))
 
 ;;;###autoload
 (defun subed-word-data-load-from-file (file)
   "Load word-level timing from FILE.
-For now, only SRV2 and JSON files are supported."
-  (interactive (list (read-file-name "JSON or srv2: "
+Supports WhisperX JSON, YouTube VTT, and Youtube SRV2 files."
+  (interactive (list (read-file-name "JSON, VTT, or srv2: "
                                      nil
                                      nil
                                      nil
@@ -130,21 +171,26 @@ For now, only SRV2 and JSON files are supported."
                                      (lambda (f)
                                        (or (file-directory-p f)
                                            (string-match
-                                            "\\.json\\'\\|\\.srv2\\'"
+                                            "\\.\\(json\\|srv2\\|vtt\\)\\'"
                                             f))))))
   (subed-word-data--load
-   (if (and (stringp file) (string-match "\\.json\\'" file))
-       (subed-word-data--extract-words-from-whisperx-json file)
-     (subed-word-data--extract-words-from-srv2 (xml-parse-file file)))))
+   (pcase (file-name-extension file)
+     ("json" (subed-word-data--extract-words-from-whisperx-json file))
+     ("srv2" (subed-word-data--extract-words-from-srv2 (xml-parse-file file)))
+     ("vtt" (subed-word-data--extract-words-from-youtube-vtt file)))))
 
 (defun subed-word-data-load-from-string (string)
   "Load word-level timing from STRING.
 For now, only JSON or SRV2 files are supported."
-  (subed-word-data--load (if (string-match "^{" string)
-              (subed-word-data--extract-words-from-whisperx-json string t)
-              (subed-word-data--extract-words-from-srv2 string))))
+  (subed-word-data--load (cond
+           ((string-match "^{" string)
+            (subed-word-data--extract-words-from-whisperx-json string t))
+           ((string-match "^WEBVTT" string)
+            (subed-word-data--extract-words-from-youtube-vtt string t))
+           (t
+            (subed-word-data--extract-words-from-srv2 string)))))
 
-(defvar subed-word-data-extensions '(".en.srv2" ".srv2") "Extensions to search for word data.")
+(defvar subed-word-data-extensions '(".en.srv2" ".srv2" ".json" ".vtt") "Extensions to search for word data.")
 
 ;;;###autoload
 (defun subed-word-data-load-maybe ()
@@ -157,8 +203,7 @@ For now, only JSON or SRV2 files are supported."
                   (setq file (concat (file-name-sans-extension (buffer-file-name)) ext))
                   (throw 'found)))
               subed-word-data-extensions))
-      (when file
-        (subed-word-data-load-from-file file)
+      (when (and file (subed-word-data-load-from-file file))
         (message "Word data loaded.")))))
 
 (defvar subed-word-data-normalizing-functions '(subed-word-data-normalize-word-default)
@@ -367,11 +412,72 @@ Requires the text properties to be set."
         (setq last-start-time nil)))
     (goto-char max-pos)))
 
+(defun subed-word-data-find-minimum-distance (from-n to-n distance-fn &optional short-circuit low-threshold)
+  "Return (number distance) that minimizes DISTANCE-FN.
+Check the range FROM (inclusive) to TO (exclusive).
+If SHORT-CIRCUIT is specified, call that function with i as the argument and stop when it returns true.
+If LOW-THRESHOLD is specified, stop when the distance is less than or equal to that number."
+  (setq low-threshold (or low-threshold 0))
+  (catch 'found
+    (let* (min-distance
+				   current-distance
+           i)
+      (setq i from-n)
+      (while (< i to-n)
+        (setq current-distance (funcall distance-fn i))
+        (when (or (null min-distance) (< current-distance (cdr min-distance)))
+          (setq min-distance (cons i current-distance))
+          (when (<= current-distance low-threshold)
+            (throw 'found min-distance)))
+        (when (and short-circuit (funcall short-circuit i))
+          (throw 'found min-distance))
+        (setq i (1+ i)))
+      min-distance)))
+
+(defun subed-word-data-find-approximate-match (phrase list-of-words &optional short-circuit)
+  "Match PHRASE against the beginning of LIST-OF-WORDS.
+LIST-OF-WORDS is a list of strings or a list of alists that have 'text.
+If SHORT-CIRCUIT is non-nil, use it as a regexp that short-circuits recognition and stops there.
+Return (distance . list of words) that minimizes the string distance from PHRASE.
+distance is expressed as a ratio of number of edits / maximum length of phrase or words.
+"
+  (let ((min-distance
+         (subed-word-data-find-minimum-distance
+          1
+          (+ (length (split-string phrase " ")) 8)
+          (lambda (num-words)
+            (let ((cand (mapconcat
+                         (lambda (o)
+                           (if (stringp o) o (alist-get 'text o)))
+                         (seq-take list-of-words num-words)
+                         " ")))
+              (/
+               (* 1.0
+                  (string-distance phrase cand))
+               (max (length phrase)
+                    (length cand)))))
+          (if (and short-circuit
+                   (not (string-match short-circuit phrase)))
+              (lambda (num-words)
+                (string-match
+                 short-circuit
+                 (mapconcat
+                  (lambda (o)
+                    (if (stringp o) o (alist-get 'text o)))
+                  (seq-take list-of-words num-words)
+                  " ")))))))
+    (cons (cdr min-distance) (seq-take list-of-words (car min-distance)))))
+
+;; (subed-word-data-find-approximate-match "Go into the room." (split-string "Go in to the room. There you will" " "))
+;; (subed-word-data-find-approximate-match "The quick brown fox jumps over the lazy dog" (split-string "The quick, oops, the quick brown fox jumps over the lazy dog and goes all sorts of places" " ") "\\<oops\\>")
+;; (subed-word-data-find-approximate-match "I already talk pretty quickly," (split-string "I already talk pretty quickly. Oops. I already talk pretty quickly, so I'm not going" " ") "\\<oops\\>")
 (defun subed-word-data-fix-subtitle-timing (beg end)
   "Sets subtitle starts and stops based on the word data.
 Assumes words haven't been edited."
   (interactive (list (if (region-active-p) (min (point) (mark)))
                      (if (region-active-p) (max (point) (mark)))))
+  (unless subed-word-data--cache
+    (call-interactively #'subed-word-data-load-from-file))
   (setq beg (or beg (point-min)))
   (setq end (if end (save-excursion
                       (goto-char end)
@@ -393,43 +499,154 @@ Assumes words haven't been edited."
                   (< (or (alist-get 'start o) 0)
                      start-ms))
                 subed-word-data--cache))
-				 min-distance
-				 current-distance
-				 current-sub
-				 num-words
-				 candidate
-				 test-distance)
+				 candidate)
     (while (and (not (> (point) end)) data)
       (setq current-sub (replace-regexp-in-string "\n" " " (subed-subtitle-text)))
-      (setq num-words (- (length (split-string current-sub " ")) 2))
-      (setq candidate (mapconcat (lambda (o) (alist-get 'text o))
-                                 (seq-take data num-words) " "))
-      (setq current-distance (string-distance current-sub candidate))
-      ;; add another word to see if the distance goes down
-      (setq test-distance (string-distance current-sub
-                                           (mapconcat (lambda (o) (alist-get 'text o))
-                                                      (seq-take data (1+ num-words)) " ")))
-      (while (< test-distance current-distance)
-        (setq current-distance test-distance
-              num-words (1+ num-words)
-              test-distance (string-distance current-sub
-                                             (mapconcat (lambda (o) (alist-get 'text o))
-                                                        (seq-take data (1+ num-words)) " "))))
-      (subed-set-subtitle-time-start
-       (alist-get
-        'start
-        (seq-find (lambda (o) (alist-get 'start o))
-                  (seq-take data num-words))))
-      (subed-set-subtitle-time-stop
-       (alist-get
-        'end
-        (seq-find (lambda (o) (alist-get 'end o))
-                  (reverse (seq-take data num-words)))))
-      (subed-word-data-refresh-text-properties-for-subtitle)
-      (setq data (seq-drop data num-words))
+      (let ((candidate (subed-word-data-find-approximate-match current-sub data)))
+        (subed-set-subtitle-time-start
+         (alist-get
+          'start
+          (seq-find (lambda (o) (alist-get 'start o))
+                    (cdr candidate))))
+        (subed-set-subtitle-time-stop
+         (alist-get
+          'end
+          (seq-find (lambda (o) (alist-get 'end o))
+                    (reverse (cdr candidate)))))
+        (subed-word-data-refresh-text-properties-for-subtitle)
+        (setq data (seq-drop data (length (cdr candidate)))))
       (unless (subed-forward-subtitle-text)
         (goto-char (point-max))))))
 
+(defun subed-word-data-move-untimed-words-from-previous ()
+	"Move untimed words from previous subtitle to current one."
+	(interactive)
+	(save-excursion
+		(subed-backward-subtitle-end)
+		(text-property-search-backward 'subed-word-data-end)
+		(goto-char (next-single-property-change (point) 'subed-word-data-end))
+		(let* ((start (point))
+					 (text (buffer-substring start (subed-jump-to-subtitle-end))))
+			(delete-region start (point))
+			(subed-forward-subtitle-text)
+			(insert text " "))))
+
+(defvar subed-word-data-script-difference-threshold 0.2
+	"*If string difference is above this threshold, include original text as a comment.")
+
+(defvar subed-word-data-oops-regexp "\\<oops\\>"
+	"*Regular expression matching the signal used after a false start.")
+
+(defun subed-word-data-combine-script-and-transcript (phrases bag-of-words &optional oops-regexp keep-transcript-words)
+	"Use PHRASES to split the words in BAG-OF-WORDS.
+If OOPS-REGEXP is non-nil, use that as the regular expression that signals a false start.
+If KEEP-TRANSCRIPT-WORDS is non-nil, don't correct transcript words.
+Return a list of subtitles and comments."
+	(let* ((phrase-length (length phrases))
+				 (phrase-cursor 0)
+				 (case-fold-search t)
+				 lookback
+				 min-candidate
+				 result)
+		(while (and (< phrase-cursor phrase-length)
+								bag-of-words)
+			(when (and oops-regexp (string-match oops-regexp (car bag-of-words)))
+				;; discard that word and figure out where we're restarting
+				(setf (elt (cdr (car result)) 3)
+              (concat (elt (cdr (car result)) 3) " " (car bag-of-words)))
+				(setf (elt (cdr (car result)) 4)
+							(string-trim
+							 (concat (or (elt (cdr (car result)) 4) "") "\n#+SKIP")))
+				(setq bag-of-words (cdr bag-of-words)))
+      (setq phrase-cursor
+						(- phrase-cursor
+							 (car (subed-word-data-find-minimum-distance
+										 0 (1+ (min phrase-cursor 4))
+										 (lambda (i)
+											 (if (< (- phrase-cursor i) 0)
+													 most-positive-fixnum
+												 (car (subed-word-data-find-approximate-match
+															 (elt phrases (- phrase-cursor i))
+															 bag-of-words
+															 oops-regexp))))))))
+      ;; mark the previous ones as oopses also
+      (dolist (o result)
+        (when (>= (car o) phrase-cursor)
+          (unless (string-match "#\\+SKIP" (or (elt (cdr o) 4) ""))
+            (setf (elt (cdr o) 4)
+                  (string-trim
+                   (concat
+                    (or (elt (cdr o) 4) "")
+                    "\n"
+                    "#+SKIP"))))))
+			(setq candidate
+						(subed-word-data-find-approximate-match
+						 (elt phrases phrase-cursor)
+						 bag-of-words
+						 oops-regexp))
+			(setq result
+						(cons
+             (cons
+              phrase-cursor
+						  (if (and oops-regexp
+											 (string-match oops-regexp (string-join (cdr candidate) " "))
+											 (not (string-match oops-regexp (elt phrases phrase-cursor))))
+								  (list nil 0 0 (string-join (cdr candidate) " ") "#+SKIP")
+							  (setq phrase-cursor (1+ phrase-cursor))
+							  (list nil 0 0
+ 										  (if (or keep-transcript-words
+                              (> (car candidate) subed-word-data-script-difference-threshold))
+												  (string-join (cdr candidate) " ")
+											  (elt phrases (1- phrase-cursor)))
+                      (cond
+											 ((and keep-transcript-words (> (car candidate) 0))
+                        (concat "#+SCRIPT: " (elt phrases (1- phrase-cursor)) "\n"
+                                "#+DISTANCE: " (format "%.2f" (car candidate))))
+											 ((= (car candidate) 0) nil)
+                       ((> (car candidate) subed-word-data-script-difference-threshold)
+											  (concat "#+SCRIPT: " (elt phrases (1- phrase-cursor)) "\n"
+                                "#+DISTANCE: " (format "%.2f" (car candidate))))
+                       ((< (car candidate) subed-word-data-script-difference-threshold)
+											  (concat "#+TRANSCRIPT: " (string-join (cdr candidate) " ") "\n"
+                                "#+DISTANCE: " (format "%.2f" (car candidate))))
+                       ))))
+						 result))
+			;; found a good match, move to the next phrase
+			(setq bag-of-words (seq-drop bag-of-words (length (cdr candidate)))))
+		(mapcar 'cdr (reverse result))))
+
+(defun subed-word-data-use-script-file (script-file output-file &optional oops-regexp keep-transcript-words)
+	"Use the info from SCRIPT-FILE to correct the current transcript.
+Write OUTPUT-FILE so that it uses the words and phrasing from SCRIPT-FILE,
+but includes any extra phrases from TRANSCRIPT-FILE (such as oopses).
+If OOPS-REGEXP is non-nil, use that as the regular expression that signals a false start.
+If KEEP-TRANSCRIPT-WORDS is non-nil, don't correct current transcript words.
+
+When called with `\\[universal-argument]', don't correct current transcript words."
+	(interactive (list
+								(read-file-name "Script: ")
+								(read-file-name "Output file: ")
+								subed-word-data-oops-regexp
+								current-prefix-arg))
+	(let* ((phrases
+					(if (string= "vtt" (file-name-extension script-file))
+							(mapcar (lambda (o) (elt o 3)) (subed-parse-file script-file))
+						(with-temp-buffer
+							(insert-file script-file)
+							(split-string (string-trim (buffer-string)) "\n"))))
+				 (bag-of-words
+					(split-string
+					 (if (derived-mode-p 'subed-mode)
+               (mapconcat (lambda (o) (elt o 3)) (subed-subtitle-list) " ")
+						 (string-trim (buffer-string)))
+					 "[ \n]+"))
+				 (result (subed-word-data-combine-script-and-transcript phrases bag-of-words oops-regexp)))
+		(subed-create-file
+		 output-file
+		 result
+		 t)
+		(find-file output-file)))
+
 (with-eval-after-load 'subed
   (add-hook 'subed-region-adjusted-hook #'subed-word-data-refresh-region))
 
