@@ -1,4 +1,4 @@
-;; -*- eval: (buttercup-minor-mode) -*-
+;; -*- lexical-binding: t; eval: (buttercup-minor-mode) -*-
 
 (load-file "./tests/undercover-init.el")
 (require 'subed-srt)
@@ -22,7 +22,7 @@ Baz.
     (subed-srt-mode)
     (progn ,@body)))
 
-(describe "SRT"
+(describe "subed-srt"
   (describe "Getting"
     (describe "the subtitle ID"
       (it "returns the subtitle ID if it can be found."
@@ -195,8 +195,27 @@ Baz.
          (insert "foo")
          (expect (subed-subtitle-relative-point) :to-equal nil)))
       )
-    )
+    (describe "the subtitle start position"
+      (it "returns the start from inside a subtitle."
+        (with-temp-srt-buffer
+         (insert mock-srt-data)
+         (re-search-backward "Bar")
+         (expect (subed-subtitle-start-pos) :to-equal 39)))
+      (it "returns the start from the beginning of the line."
+        (with-temp-srt-buffer
+         (insert mock-srt-data)
+         (re-search-backward "^2\n")
+         (expect (subed-subtitle-start-pos) :to-equal (point)))))
 
+    )
+  (describe "Converting to msecs"
+    (it "works with numbers."
+      (expect (with-temp-srt-buffer (subed-to-msecs 5123)) :to-equal 5123))
+    (it "works with numbers as strings."
+      (expect (with-temp-srt-buffer (subed-to-msecs "5123")) :to-equal 5123))
+    (it "works with timestamps."
+      (with-temp-srt-buffer
+       (expect (subed-to-msecs "00:00:05,124") :to-equal 5124))))
   (describe "Jumping"
     (describe "to current subtitle ID"
       (it "returns ID's point when point is already on the ID."
@@ -413,7 +432,19 @@ Baz.
          (backward-char)
          (expect (subed-jump-to-subtitle-end) :to-be 96)
          (expect (looking-at "^$") :to-be t)))
-      )
+      (it "works with multi-line cues where a line is all numbers."
+        (with-temp-srt-buffer
+         (insert "1
+00:00:00,000 --> 00:00:01,000
+This is first subtitle.
+123456789
+
+2
+00:00:01,000 --> 00:00:02,000
+This is second subtitle.
+")
+         (goto-char (point-min))
+         (expect (subed-jump-to-subtitle-end) :to-be 66))))
     (describe "to next subtitle ID"
       (it "returns point when there is a next subtitle."
         (with-temp-srt-buffer
@@ -624,7 +655,7 @@ Baz.
       (with-temp-srt-buffer
        (insert mock-srt-data)
        (subed-jump-to-subtitle-end 2)
-       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 400))
+       (subed-set-subtitle-time-start (subed-timestamp-to-msecs "1:02:03,400") nil t t)
        (expect (buffer-string) :to-equal (concat "1\n"
                                                  "00:01:01,000 --> 00:01:05,123\n"
                                                  "Foo.\n\n"
@@ -634,7 +665,7 @@ Baz.
                                                  "3\n"
                                                  "00:03:03,45 --> 00:03:15,5\n"
                                                  "Baz.\n"))
-       (subed-set-subtitle-time-stop (+ (* 5 60 60 1000) (* 6 60 1000) (* 7 1000) 800))
+       (subed-set-subtitle-time-stop (subed-timestamp-to-msecs "5:06:07,800") nil t t)
        (expect (buffer-string) :to-equal (concat "1\n"
                                                  "00:01:01,000 --> 00:01:05,123\n"
                                                  "Foo.\n\n"
@@ -648,7 +679,7 @@ Baz.
       (with-temp-srt-buffer
        (insert mock-srt-data)
        (subed-jump-to-subtitle-time-stop 3)
-       (subed-set-subtitle-time-start (+ (* 2 60 60 1000) (* 4 60 1000) (* 6 1000) 800) 1)
+       (subed-set-subtitle-time-start (+ (* 2 60 60 1000) (* 4 60 1000) (* 6 1000) 800) 1 t t)
        (expect (buffer-string) :to-equal (concat "1\n"
                                                  "02:04:06,800 --> 00:01:05,123\n"
                                                  "Foo.\n\n"
@@ -659,7 +690,7 @@ Baz.
                                                  "00:03:03,45 --> 00:03:15,5\n"
                                                  "Baz.\n"))
        (subed-jump-to-subtitle-text 1)
-       (subed-set-subtitle-time-stop (+ (* 3 60 60 1000) (* 5 60 1000) (* 7 1000) 900) 3)
+       (subed-set-subtitle-time-stop (+ (* 3 60 60 1000) (* 5 60 1000) (* 7 1000) 900) 3 t t)
        (expect (buffer-string) :to-equal (concat "1\n"
                                                  "02:04:06,800 --> 00:01:05,123\n"
                                                  "Foo.\n\n"
@@ -673,10 +704,10 @@ Baz.
       (with-temp-srt-buffer
        (insert mock-srt-data)
        (subed-jump-to-subtitle-id 3)
-       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 4) 3)
+       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 4) 3 t)
        (expect (save-excursion (subed-jump-to-subtitle-time-start)
                                (thing-at-point 'line)) :to-equal "01:02:03,004 --> 00:03:15,5\n")
-       (subed-set-subtitle-time-stop (+ (* 2 60 60 1000) (* 3 60 1000) (* 4 1000) 60) 3)
+       (subed-set-subtitle-time-stop (+ (* 2 60 60 1000) (* 3 60 1000) (* 4 1000) 60) 3 t)
        (expect (save-excursion (subed-jump-to-subtitle-time-start)
                                (thing-at-point 'line)) :to-equal "01:02:03,004 --> 02:03:04,060\n")))
     )
@@ -1523,9 +1554,50 @@ Baz.
        (insert mock-srt-data)
        (subed-jump-to-subtitle-text 2)
        (subed-merge-with-next)
-       (expect (subed-subtitle-text) :to-equal "Bar.\nBaz.")
+       (expect (subed-subtitle-text) :to-equal "Bar. Baz.")
        (expect (subed-subtitle-msecs-start) :to-equal 122234)
-       (expect (subed-subtitle-msecs-stop) :to-equal 195500))))
+       (expect (subed-subtitle-msecs-stop) :to-equal 195500)))
+    (it "handles lines that are all numbers."
+      (with-temp-srt-buffer
+       (insert "1
+00:00:00,000 --> 00:00:01,000
+This is first subtitle.
+
+2
+00:00:01,000 --> 00:00:02,000
+This is the second subtitle.
+
+3
+00:00:02,000 --> 00:00:03,000
+This is the third subtitle.
+
+4
+00:00:03,000 --> 00:00:05,000
+This is the fourth subtitle.
+123456789
+
+5
+00:00:05,000 --> 00:00:06,000
+This is the sixth subtitle.")
+       (re-search-backward "This is the fourth subtitle")
+       (subed-merge-with-next)
+       (expect (buffer-string) :to-equal
+               "1
+00:00:00,000 --> 00:00:01,000
+This is first subtitle.
+
+2
+00:00:01,000 --> 00:00:02,000
+This is the second subtitle.
+
+3
+00:00:02,000 --> 00:00:03,000
+This is the third subtitle.
+
+4
+00:00:03,000 --> 00:00:06,000
+This is the fourth subtitle.
+123456789 This is the sixth subtitle."))))
 
   (describe "A comment"
     (it "is validated."
